==> portmanager-0.3.4.tar.gz: portmanager-0.3.4/libMGPM/src/MGPMrSinglePortCreateStatusDb.c <==
/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMGPM.h>

int	rSinglePortUpgrade( structProperty* property, int forced );
int	rDependencyParse( structProperty* property, char* buffer, char* availablePortName, char* availablePortDir, char* dependType, int* counterPtr );
int	rFillDependencies( structProperty* property, char* availablePortName, char* availablePortDir, int* counterPtr );
int	rGetPortName( structProperty* property, char* portDir, char* portName );
int	rParse( structProperty* property, char* availablePortName, char* availablePortDir, char* dependType, char* buffer, int* counterPtr );

int	MGPMrSinglePortCreateStatusDb( structProperty* property, char* portDir )
{
	FILE*	dependencyDbStream	= NULL;
	char	exact[]			= "exact";
	char	id[]			= "MGPMrSinglePortCreateStatusDb";
	char*	availablePortDir	= NULL;
	char*	availablePortName	= NULL;
	char*	availablePortsDbFlagPtr	= NULL;
	int	availablePortsDbIDX	= 0;
	int	bufferSize		= 0xffff;
	int	counter			= 0;
	int	errorCode		= 0;
	int	idx			= 0;
	int*	counterPtr		= NULL;

	counterPtr	= &counter;

	availablePortDir	= (char*)calloc( bufferSize, 1 );
	availablePortName	= (char*)calloc( bufferSize, 1 );

	/*.............................................................*/
	fprintf( stdout,"%s\n", SINGLE_LINES );
	if( property->forced )
	{
		fprintf( stdout,"%s %s: Collecting installed port data \"forced mode\"\n", "portmanager", PACKAGE_VERSION );
	}
	else
	{
		fprintf( stdout,"%s %s: Collecting installed port data \n", "portmanager", PACKAGE_VERSION );

	}
	fprintf( stdout,"%s\n", SINGLE_LINES );
	while( fflush( stdout ) );
	strcpy( availablePortDir, portDir );

	if( property->optionsChanged == 0 )	/* set by MGPMrUpdate for example to indicate a ports's options just changed */
	{
		property->availablePortsDb	= MGdbCreate(
							property->availablePortsDbFileName,
							property->fieldAvailablePortsDbPortName,
							property->fieldAvailablePortsDbPortDir,
							property->fieldAvailablePortsDbFlag,
							NULL);

		dependencyDbStream	= fopen( property->availableDependenciesDbFileName, "w" );
		if( dependencyDbStream == 0 )
		{
			fprintf( stdout, "%s %s error: could not open %s\n", id, PACKAGE_VERSION, property->availableDependenciesDbFileName );
			perror( "system message" );
			return( 1 );
		}

		/*
		 * creates dependencyDb with header record, same as MGdbCreate.  In this case records will be added by
		 * fprint'ing to dependencyDbStream instead of using MGdbAdd because speed is important here so we don't
		 * want a dependencyDb object yet.
		 */
		fprintf( dependencyDbStream, "%s%c%s%c%s%c%s%c%s%c%c", "fieldAvailableDependenciesDbPortDir", 0, "fieldAvailableDependenciesDbPortName", 0,
			"fieldAvailableDependenciesDbDependencyDir", 0,	"fieldAvailableDependenciesDbDependencyName", 0,
			"fieldAvailableDependenciesDbDependType", 0, LINEFEED );
		while( fflush( stdout ) );
		fclose( dependencyDbStream );

		property->availableDependenciesDb	= MGdbOpen( property->availableDependenciesDbFileName );
		property->configDb			= MGdbOpen( property->configDbFileName );
	}
		 
	if( ( errorCode = rGetPortName( property, availablePortDir, availablePortName ) ) )
	{
		fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "rGetPortName", errorCode );
		return( 1 );
	}

	MGdbGoTop( property->availablePortsDb );
	if( MGdbSeek( property->availablePortsDb,
			property->fieldAvailablePortsDbPortDir,
			availablePortDir,
			property->fieldAvailablePortsDbPortDir,
			exact ) )
	{
	}
	else
	{
		MGdbAdd( property->availablePortsDb, availablePortName, availablePortDir, "0", NULL);

	}

	if( ( errorCode = rFillDependencies( property, availablePortName, availablePortDir, counterPtr ) ) )
	{
		fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "rFillDependencies", errorCode );
		return( 1 );
	}

	/*
 	 * fill dependencies for each record in property->availablePortsDb
 	 */		
	availablePortsDbIDX	= MGdbGetRecordQty( property->availablePortsDb );
	idx	= 1;
	while( idx < availablePortsDbIDX )
	{
		availablePortsDbFlagPtr	= MGdbGet( property->availablePortsDb, idx, property->fieldAvailablePortsDbFlag );
		if( availablePortsDbFlagPtr && strncmp( availablePortsDbFlagPtr, "0", 1 ) == 0 )
		{
			strcpy( availablePortDir, MGdbGet( property->availablePortsDb, idx, property->fieldAvailablePortsDbPortDir ) );
			strcpy( availablePortName, MGdbGet( property->availablePortsDb, idx, property->fieldAvailablePortsDbPortName ) );
			rFillDependencies( property, availablePortName,	availablePortDir, counterPtr );
			availablePortsDbIDX	= MGdbGetRecordQty( property->availablePortsDb );
			idx	= 0;
		}
		idx++;
	}
	if( property->optionsChanged == 0 )	/* set by MGPMrUpdate for example to indicate a ports's options just changed */
	{
		MGdbDestroy( property->availablePortsDb );
		MGdbDestroy( property->availableDependenciesDb );
		MGdbDestroy( property->configDb );
	}
	free( availablePortDir );
	free( availablePortName );
	return( 0 );
}

int	rGetPortName( structProperty* property, char* portDir, char* portName )
{
	FILE*	makeStream			= NULL;
	char	exact[]				= "exact";
	char	id[]				= "rGetPortName";
	char*	availablePortsDbPortNamePtr	= NULL;
	char*	charPtr				= NULL;
	char*	command				= NULL;
	char*	optionsPtr			= NULL;
	int	bufferSize			= 0xffff;

	command		= (char*)calloc( bufferSize, 1 );

	MGdbGoTop( property->availablePortsDb );
	availablePortsDbPortNamePtr	= MGdbSeek( property->availablePortsDb, property->fieldAvailablePortsDbPortDir, portDir,
						property->fieldAvailablePortsDbPortName, exact );

	if( availablePortsDbPortNamePtr == NULL )
	{
		/*
		 * see if this port has anything special in configDb
		 */
		MGdbGoTop( property->configDb );
		optionsPtr	= MGdbSeek( property->configDb, property->fieldConfigDbKey, portDir, property->fieldConfigDbValue, exact );
		strcpy( command, "cd " );
		strcat( command, PORTSDIR );
		strcat( command, portDir );
		strcat( command, "; make " );
		strcat( command, " -V PKGNAME " );
		if( optionsPtr )
		{
			strcat( command, optionsPtr );
		}
		makeStream	= popen( command, "r" );
		if( makeStream == 0 )
		{
			fprintf( stdout, "%s error: reading make -V PKGNAME into availablePortName\n", id );
			perror( "system message" );
			return( 1 );
		}
		portName[0]	= 0;
		fread( portName, bufferSize-1, 1, makeStream );
		if( portName == NULL )
		{
			fprintf( stdout, "%s error: reading make -V PKGNAME into availablePortName\n", id );
			perror( "system message" );
			return( 1 );
		}
		charPtr	= portName;
		while( charPtr < portName + (bufferSize-1) )
		{
			if( charPtr[0]	== SPACE || charPtr[0] == TAB || charPtr[0] == LINEFEED || charPtr[0] == ':' )
			{
				charPtr[0]	= 0;
				break;
			}
			charPtr++;
		}
		pclose( makeStream );
	}
	else
	{
		MGdbGoTop( property->availablePortsDb );
		strcpy( portName,  MGdbSeek( property->availablePortsDb, property->fieldAvailablePortsDbPortDir, portDir,
					property->fieldAvailablePortsDbPortName, exact ) );
	}
	free( command );
	return( 0 );
}

int	rFillDependencies( structProperty* property, char* availablePortName, char* availablePortDir, int* counterPtr )
{
	char	BUILD_DEPENDS[]		= "BUILD_DEPENDS";
	char	RUN_DEPENDS[]		= "RUN_DEPENDS";
	char	exact[]			= "exact";
	char	id[]			= "rFillDependencies";
	char*	availablePortDirPtr	= NULL;
	char*	buffer			= NULL;
	int	availablePortsDbIDX	= 0;
	int	bufferSize		= 0xffff;

	buffer			= calloc( bufferSize + 1, 1 );
	rDependencyParse( property, buffer, availablePortName, availablePortDir, BUILD_DEPENDS, counterPtr );
	buffer[0]	= 0;
	rDependencyParse( property, buffer, availablePortName, availablePortDir, RUN_DEPENDS, counterPtr );
	free( buffer );

	/*
	 * Flag availablePortsDb.availablePortName as "1" after dependencies added
	 */
	if( strlen(availablePortDir) < 3 )
	{
		fprintf( stdout, "%s error: strlen(availablePortDir)-=>%d\n", id, strlen(availablePortDir) );
		return( 1 );
	}
	MGdbGoTop( property->availablePortsDb );
	availablePortDirPtr	= MGdbSeek( property->availablePortsDb, property->fieldAvailablePortsDbPortDir, availablePortDir,
					property->fieldAvailablePortsDbPortDir, exact );
	availablePortsDbIDX	= MGdbGetRecno( property->availablePortsDb ) - 1;
	property->availablePortsDb = MGdbDelete( property->availablePortsDb, availablePortsDbIDX );
	MGdbAdd( property->availablePortsDb, availablePortName, availablePortDir, "1", NULL);
	return( 0 );
}

int	rDependencyParse( structProperty* property, char* buffer, char* availablePortName, char* availablePortDir, char* dependType, int* counterPtr )
{
	FILE*	makeStream	= NULL;
	char	exact[]		= "exact";
	char	id[]		= "rDependencyParse";
	char*	command		= NULL;
	char*	optionsPtr	= NULL;
	int	bufferSize	= 0xffff;

	/*
	 * port make config option WITH_BUILD_DEPENDS_ARE_LEAVES set.
	 * so not traking build dependencies
	 */
	if( property->buildDependsAreLeaves && strcmp( dependType, "BUILD_DEPENDS" ) == 0 )
	{
		return( 0 );
	}

	command	= (char*)calloc( bufferSize, 1 );
	strcpy( command, "cd " );
	strcat( command, PORTSDIR );
	strcat( command, availablePortDir );
	strcat( command, "; make -V " );
	if( strcmp( dependType, "BUILD_DEPENDS" ) == 0 )
	{
		strcat( command, "BUILD_DEPENDS -V EXTRACT_DEPENDS -V PATCH_DEPENDS -V FETCH_DEPENDS " );
	}
	else
	{
		strcat( command, "RUN_DEPENDS -V LIB_DEPENDS -V DEPENDS " );
	}
	/*
	 * see if this port has anything special in configDb
	 */
	MGdbGoTop( property->configDb );
	optionsPtr	= MGdbSeek( property->configDb, property->fieldConfigDbKey, availablePortDir, property->fieldConfigDbValue, exact );
	if( optionsPtr )
	{
		strcat( command, optionsPtr );
	}
	makeStream	= popen( command, "r" );
	if( makeStream == 0 )
	{
		fprintf( stdout, "%s error: reading %s into buffer\n", id, command );
		perror( "system message" );
		return( 1 );
	}	
	fread( buffer, bufferSize-1, 1, makeStream );
	if( strlen( buffer ) > 1 )
	{
		rParse( property, availablePortName, availablePortDir, dependType, buffer, counterPtr );
	}
	pclose( makeStream );
	free( command );
	return( 0 );
}

int	rParse(  structProperty* property, char* availablePortName, char* availablePortDir, char* dependType, char* buffer, int* counterPtr )
{
	FILE*	dependencyDbStream	= NULL;
	char	exact[]			= "exact";
	char	id[]			= "rParse";
	char*	dependencyPortDir	= NULL;
	char*	dependencyPortName	= NULL;
	char*	optionsPtr		= NULL;
	int	bufferSize		= 0xffff;
	int	idx1			= 0;
	int	idx2			= 0;

	dependencyPortDir	= (char*)calloc( bufferSize, 1 );
	dependencyPortName	= (char*)calloc( bufferSize, 1 );

	bufferSize	= strlen( buffer );
	idx1		= 0;
	idx2		= 0;

	if( strcmp( dependType, "RUN_DEPENDS" ) == 0 )
	{
		counterPtr[0]++;

		MGdbGoTop( property->configDb );
		optionsPtr	= MGdbSeek( property->configDb, property->fieldConfigDbKey, availablePortDir, property->fieldConfigDbValue, exact );
		if( optionsPtr )
		{
			fprintf( stdout, "%05d %s %s %s\n", counterPtr[0], availablePortName, availablePortDir, optionsPtr );
			while( fflush( stdout ) );
		}
		else
		{
			fprintf( stdout, "%05d %s %s\n", counterPtr[0], availablePortName, availablePortDir );
		}
		optionsPtr	= NULL;
	}

	dependencyDbStream	= fopen( property->availableDependenciesDbFileName, "a" );

	if( dependencyDbStream == 0 )
	{
		fprintf( stdout, "%s %s error: could not open %s\n", id, PACKAGE_VERSION, property->availableDependenciesDbFileName );
		perror( "system message" );
		return( 1 );
	}
	while( idx1 < bufferSize )
	{
		if( buffer[idx1] == ':' )
		{
			idx1++;
			idx2	= idx1;
			while( idx2 < bufferSize && buffer[idx2] != ' ' && buffer[idx2] != LINEFEED && buffer[idx2] != TAB && buffer[idx2] != ':' )
			{
				idx2++;
			}
			buffer[idx2]		= 0;
			if( idx1 + 11 < bufferSize )
			{
				strcpy( dependencyPortDir, buffer + idx1 + 10 );
			}
			else
			{
				assert(0);
			}
			rGetPortName( property, dependencyPortDir, dependencyPortName );
			/*
			 * add dependency to availableDependenciesDb may contain dupes
			 */
			if( property->optionsChanged )
			{
				MGdbAdd( property->availableDependenciesDb,
					availablePortDir,
					availablePortName,
					dependencyPortDir,
					dependencyPortName,
					dependType,
					NULL );

				MGdbGoTop( property->configDb );
				optionsPtr =	MGdbSeek( property->configDb,
							property->fieldConfigDbKey,
							dependencyPortDir,
							property->fieldConfigDbValue,
							exact );
				if( optionsPtr )
				{
					fprintf( stdout, "\t%s %s %s\n", dependencyPortName, dependencyPortDir, optionsPtr );
				}
				else
				{
					fprintf( stdout, "\t%s %s\n", dependencyPortName, dependencyPortDir );
				}
				while( fflush( stdout ) );

			}
			else
			{
				fprintf( dependencyDbStream, "%s%c%s%c%s%c%s%c%s%c%c", availablePortDir, 0, availablePortName, 0, dependencyPortDir,
					0, dependencyPortName, 0, dependType, 0, LINEFEED );
			}
			/*
			 * if dependency not in availablePortsDb add it
			 */
			MGdbGoTop( property->availablePortsDb );
			if( MGdbSeek( property->availablePortsDb, property->fieldAvailablePortsDbPortDir, dependencyPortDir,
				property->fieldAvailablePortsDbPortDir, exact ) == 0 )
			{
				MGdbAdd( property->availablePortsDb, dependencyPortName, dependencyPortDir, "0", NULL);
			}
		}
		idx1++;
	}	
	fclose( dependencyDbStream );
	free( dependencyPortDir );
	free( dependencyPortName );
	return(0);
}

==> portmanager-0.4.1.tar.gz: portmanager-0.4.1/libMGPM/src/MGPMrSinglePortCreateStatusDb.c <==
/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMGPM.h>

int	rSinglePortUpgrade( structProperty* property, int forced );
int	rFillDependencies( structProperty* property, char* availablePortName, char* availablePortDir, int* counterPtr );

int	MGPMrSinglePortCreateStatusDb( structProperty* property, char* portDir )
{
	FILE*	dependencyDbStream	= NULL;
	FILE*	pHandle			= NULL;
	char	exact[]			= "exact";
	char	id[]			= "MGPMrSinglePortCreateStatusDb";
	char*	availablePortDir	= NULL;
	char*	availablePortName	= NULL;
	char*	availablePortsDbFlagPtr	= NULL;
	char*	buffer			= NULL;
	char*	command			= NULL;
	int	availablePortsDbIDX	= 0;
	int	bufferSize		= 0xffff;
	int	counter			= 0;
	int	errorCode		= 0;
	int	idx			= 0;
	int*	counterPtr		= NULL;

	counterPtr	= &counter;

	availablePortDir	= (char*)calloc( 511, 1 );
	availablePortName	= (char*)calloc( 511, 1 );
	buffer			= (char*)calloc( bufferSize, 1 );
	command			= (char*)calloc( 511, 1 );

	/*.............................................................*/
	if( property->pmMode != 2 )
	{
		fprintf( stdout,"%s\n", SINGLE_LINES );
		if( property->forced )
		{
			fprintf( stdout,"%s %s: Collecting installed port data \"forced mode\"\n", "portmanager", PACKAGE_VERSION );
		}
		else
		{
			fprintf( stdout,"%s %s: Collecting installed port data \n", "portmanager", PACKAGE_VERSION );

		}
		fprintf( stdout,"%s\n", SINGLE_LINES );
		while( fflush( stdout ) );
	}
	MGmStrcpy( availablePortDir, portDir );

	if( property->optionsChanged == 0 )	/* set by MGPMrUpdate for example to indicate a ports's options just changed */
	{
		property->availablePortsDb	= MGdbCreate(
							property->availablePortsDbFileName,
							property->fieldAvailablePortsDbPortName,
							property->fieldAvailablePortsDbPortDir,
							property->fieldAvailablePortsDbFlag,
							NULL);

		MGmFopen( dependencyDbStream, property->availableDependenciesDbFileName, "w" );
		if( dependencyDbStream == 0 )
		{
			fprintf( stderr, "%s %s error: could not open %s\n", id, PACKAGE_VERSION, property->availableDependenciesDbFileName );
			while( fflush( stderr ) );
			perror( "system message" );
			return( 1 );
		}

		/*
		 * creates dependencyDb with header record, same as MGdbCreate.  In this case records will be added by
		 * fprint'ing to dependencyDbStream instead of using MGdbAdd because speed is important here so we don't
		 * want a dependencyDb object yet.
		 */
		fprintf( dependencyDbStream, "%s%c%s%c%s%c%s%c%s%c%c", "fieldAvailableDependenciesDbPortDir", 0, "fieldAvailableDependenciesDbPortName", 0,
			"fieldAvailableDependenciesDbDependencyDir", 0,	"fieldAvailableDependenciesDbDependencyName", 0,
			"fieldAvailableDependenciesDbDependType", 0, LINEFEED );
		while( fflush( stdout ) );
		fclose( dependencyDbStream );

		property->availableDependenciesDb	= MGdbOpen( property->availableDependenciesDbFileName );
		property->configDb			= MGdbOpen( property->configDbFileName );
		property->installedPortsDb		= MGdbOpen( property->installedPortsDbFileName );

		if( MGrIfFileExist( property->ignoreDbFileName ) )
		{
			property->ignoreDb	= MGdbOpen( property->ignoreDbFileName );
		}
		else
		{
			property->ignoreDb	= MGdbCreate( property->ignoreDbFileName,
								property->fieldignoreDbPortDir,
								property->fieldIgnoreDbReason, NULL );
		}
	}

		 
	if( property->pmMode != 2 && ( errorCode = MGPMrGetPortName( property, availablePortDir, availablePortName ) ) )
	{
		fprintf( stderr, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrGetPortName", errorCode );
		while( fflush( stderr ) );
		return( 1 );
	}
	else
	{
	
		MGmStrcpy( command, "cd " );
		MGmStrcat( command, PORTSDIR );
		MGmStrcat( command, availablePortDir );
		MGmStrcat( command, "; make -V PKGNAME" );
		pHandle	= popen( command, "r" );
		fread( buffer, 0xffff - 1, 1, pHandle );
		pclose( pHandle );
		idx	= 0;
		while( idx < 0xffff )
		{
			if( buffer[idx] == LINEFEED || buffer[idx] == TAB || buffer[idx] == SPACE )
			{
				buffer[idx]	= 0;
				break;
			}
			idx++;
		}
		MGmStrcpy( availablePortName, buffer );
/*
fprintf( stdout, "DEBUG: availablePortName-=>%s\n", availablePortName );		
*/
	}

	MGdbGoTop( property->availablePortsDb );
	if( MGdbSeek( property->availablePortsDb,
			property->fieldAvailablePortsDbPortDir,
			availablePortDir,
			property->fieldAvailablePortsDbPortDir,
			exact ) )
	{
	}
	else
	{
		MGdbAdd( property->availablePortsDb, availablePortName, availablePortDir, "0", NULL);

	}

	if( ( errorCode = rFillDependencies( property, availablePortName, availablePortDir, counterPtr ) ) )
	{
		fprintf( stderr, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "rFillDependencies", errorCode );
		while( fflush( stderr ) );
		return( 1 );
	}

	/*
 	 * fill dependencies for each record in property->availablePortsDb
 	 */		
	availablePortsDbIDX	= MGdbGetRecordQty( property->availablePortsDb );
	idx	= 1;
	while( idx < availablePortsDbIDX )
	{
		availablePortsDbFlagPtr	= MGdbGet( property->availablePortsDb, idx, property->fieldAvailablePortsDbFlag );
		if( availablePortsDbFlagPtr && strncmp( availablePortsDbFlagPtr, "0", 1 ) == 0 )
		{
			MGmStrcpy( availablePortDir, MGdbGet( property->availablePortsDb, idx, property->fieldAvailablePortsDbPortDir ) );
			MGmStrcpy( availablePortName, MGdbGet( property->availablePortsDb, idx, property->fieldAvailablePortsDbPortName ) );
			rFillDependencies( property, availablePortName,	availablePortDir, counterPtr );
			availablePortsDbIDX	= MGdbGetRecordQty( property->availablePortsDb );
			idx	= 0;
		}
		idx++;
	}
	if( property->optionsChanged == 0 )	/* set by MGPMrUpdate for example to indicate a ports's options just changed */
	{
		MGdbDestroy( property->availableDependenciesDb );
		MGdbDestroy( property->availablePortsDb );
		MGdbDestroy( property->configDb );
		MGdbDestroy( property->ignoreDb );
		MGdbDestroy( property->installedPortsDb );
	}

	free( availablePortDir );
	free( availablePortName );
	free( buffer );
	free( command );
	return( 0 );
}

int	rFillDependencies( structProperty* property, char* availablePortName, char* availablePortDir, int* counterPtr )
{
	char	BUILD_DEPENDS[]		= "BUILD_DEPENDS";
	char	RUN_DEPENDS[]		= "RUN_DEPENDS";
	char	exact[]			= "exact";
	char	id[]			= "rFillDependencies";
	char*	availablePortDirPtr	= NULL;
	char*	buffer			= NULL;
	int	availablePortsDbIDX	= 0;
	int	bufferSize		= 0xffff;

	buffer			= calloc( bufferSize + 1, 1 );

	if( MGPMrDependencyParse( property, buffer, availablePortName, availablePortDir, BUILD_DEPENDS, counterPtr ) )
	{
		fprintf( stderr, "%s %s error: %s returned an error\n", id, PACKAGE_VERSION, "rMGPMrDependencyParse" );
		while( fflush( stderr ) );
		free( buffer );
		return( 1 );
	}

	buffer[0]	= 0;

	if( MGPMrDependencyParse( property, buffer, availablePortName, availablePortDir, RUN_DEPENDS, counterPtr ) )
	{
		fprintf( stderr, "%s %s error: %s returned an error\n", id, PACKAGE_VERSION, "rMGPMrDependencyParse" );
		while( fflush( stderr ) );
		free( buffer );
		return( 1 );
	}

	free( buffer );

	/*
	 * Flag availablePortsDb.availablePortName as "1" after dependencies added
	 */
	if( MGrStrlen(availablePortDir) < 3 )
	{
		fprintf( stderr, "%s error: MGrStrlen(availablePortDir)-=>%d\n", id, MGrStrlen(availablePortDir) );
		while( fflush( stderr ) );
		return( 1 );
	}
	MGdbGoTop( property->availablePortsDb );
	availablePortDirPtr	= MGdbSeek( property->availablePortsDb, property->fieldAvailablePortsDbPortDir, availablePortDir,
					property->fieldAvailablePortsDbPortDir, exact );
	availablePortsDbIDX	= MGdbGetRecno( property->availablePortsDb ) - 1;
	property->availablePortsDb = MGdbDelete( property->availablePortsDb, availablePortsDbIDX );
	MGdbAdd( property->availablePortsDb, availablePortName, availablePortDir, "1", NULL);
	return( 0 );
}
